@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2003 Sun Microsystems, Inc.
+ * Copyright (C) 2004 Red Hat Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -33,6 +34,11 @@
 #include <libgnome/gnome-i18n.h>
 
 #include <sys/ioctl.h>
+#ifdef HAVE_SYS_SOCKIO_H
+#include <sys/sockio.h>
+#endif /* HAVE_SYS_SOCKIO_H */
+#include <sys/param.h>
+#include <sys/socket.h>
 #include <net/if.h>
 #include <net/if_arp.h>
 #include <arpa/inet.h>
@@ -52,6 +58,8 @@ enum
   PROP_0,
   PROP_NAME,
   PROP_STATE,
+  PROP_WIRELESS,
+  PROP_SIGNAL_STRENGTH,
   PROP_ERROR
 };
 
@@ -64,12 +72,14 @@ struct _NetstatusIfacePrivate
   long            out_packets;
   long            in_bytes;
   long            out_bytes;
+  int             signal_strength;
   GError         *error;
 
   int             sockfd;
   guint           monitor_id;
 
   guint           error_polling : 1;
+  guint           is_wireless : 1;
 };
 
 static void     netstatus_iface_instance_init   (NetstatusIface      *iface,
@@ -156,6 +166,24 @@ netstatus_iface_class_init (NetstatusIfaceClass *klass)
 						      NETSTATUS_STATE_DISCONNECTED,
 						      G_PARAM_READABLE));
 
+  g_object_class_install_property (gobject_class,
+				   PROP_WIRELESS,
+				   g_param_spec_boolean ("wireless",
+							 _("Wireless"),
+							 _("Whether the interface is a wireless interface"),
+							 FALSE,
+							 G_PARAM_READABLE));
+
+  g_object_class_install_property (gobject_class,
+				   PROP_SIGNAL_STRENGTH,
+				   g_param_spec_int ("signal-strength",
+						     _("Signal"),
+						     _("Wireless signal strength percentage"),
+						     0,
+						     100,
+						     0,
+						     G_PARAM_READABLE));
+
   g_object_class_install_property (gobject_class,
 				   PROP_ERROR,
 				   g_param_spec_boxed ("error",
@@ -229,6 +257,12 @@ netstatus_iface_get_property (GObject    *object,
     case PROP_STATE:
       g_value_set_enum (value, iface->priv->state);
       break;
+    case PROP_WIRELESS:
+      g_value_set_boolean (value, iface->priv->is_wireless);
+      break;
+    case PROP_SIGNAL_STRENGTH:
+      g_value_set_int (value, iface->priv->signal_strength);
+      break;
     case PROP_ERROR:
       g_value_set_boxed (value, iface->priv->error);
     default:
@@ -312,6 +346,22 @@ netstatus_iface_get_byte_statistics (NetstatusIface *iface,
     *out = iface->priv->out_bytes;
 }
 
+gboolean
+netstatus_iface_get_is_wireless (NetstatusIface *iface)
+{
+  g_return_val_if_fail (NETSTATUS_IS_IFACE (iface), FALSE);
+
+  return iface->priv->is_wireless;
+}
+
+int
+netstatus_iface_get_signal_strength (NetstatusIface *iface)
+{
+  g_return_val_if_fail (NETSTATUS_IS_IFACE (iface), 0);
+
+  return iface->priv->signal_strength;
+}
+
 void
 netstatus_iface_set_error (NetstatusIface *iface,
 			   const GError   *error)
@@ -501,6 +551,32 @@ netstatus_iface_poll_state (NetstatusIface *iface)
   return state;
 }
 
+static gboolean
+netstatus_iface_poll_wireless_details (NetstatusIface *iface,
+				       int            *signal_strength)
+{
+  char     *error_message;
+  gboolean  is_wireless;
+
+  error_message = netstatus_sysdeps_read_iface_wireless_details (iface->priv->name,
+								 &is_wireless,
+								 signal_strength);
+
+  if (error_message)
+    {
+      netstatus_iface_set_polling_error (iface,
+					 NETSTATUS_ERROR_WIRELESS_DETAILS,
+					 error_message);
+      g_free (error_message);
+      
+      return FALSE;
+    }
+
+  netstatus_iface_clear_error (iface, NETSTATUS_ERROR_WIRELESS_DETAILS);
+
+  return is_wireless;
+}
+
 static void
 netstatus_iface_increase_poll_delay_in_error (NetstatusIface *iface)
 {
@@ -539,6 +615,8 @@ static gboolean
 netstatus_iface_monitor_timeout (NetstatusIface *iface)
 {
   NetstatusState state;
+  int            signal_strength;
+  gboolean       is_wireless;
  
   state = netstatus_iface_poll_state (iface);
 
@@ -549,6 +627,19 @@ netstatus_iface_monitor_timeout (NetstatusIface *iface)
       g_object_notify (G_OBJECT (iface), "state");
     }
 
+  is_wireless = netstatus_iface_poll_wireless_details (iface, &signal_strength);
+  if (iface->priv->is_wireless != is_wireless)
+    {
+      iface->priv->is_wireless = is_wireless;
+      g_object_notify (G_OBJECT (iface), "wireless");
+    }
+
+  if (iface->priv->signal_strength != signal_strength)
+    {
+      iface->priv->signal_strength = signal_strength;
+      g_object_notify (G_OBJECT (iface), "signal-strength");
+    }
+
   netstatus_iface_increase_poll_delay_in_error (iface);
   
   return TRUE;
@@ -557,10 +648,18 @@ netstatus_iface_monitor_timeout (NetstatusIface *iface)
 static void
 netstatus_iface_init_monitor (NetstatusIface *iface)
 {
-  iface->priv->in_packets  = 0;
-  iface->priv->out_packets = 0;
-  iface->priv->in_bytes    = 0;
-  iface->priv->out_bytes   = 0;
+  iface->priv->in_packets      = 0;
+  iface->priv->out_packets     = 0;
+  iface->priv->in_bytes        = 0;
+  iface->priv->out_bytes       = 0;
+  iface->priv->signal_strength = 0;
+  iface->priv->is_wireless     = FALSE;
+
+  g_object_freeze_notify (G_OBJECT (iface));
+  g_object_notify (G_OBJECT (iface), "state");
+  g_object_notify (G_OBJECT (iface), "wireless");
+  g_object_notify (G_OBJECT (iface), "signal-strength");
+  g_object_thaw_notify (G_OBJECT (iface));
 
   if (iface->priv->monitor_id)
     {
@@ -646,7 +745,7 @@ netstatus_iface_get_inet4_details (NetstatusIface  *iface,
   strncpy (if_req.ifr_name, iface->priv->name, IF_NAMESIZE - 1);
   if_req.ifr_name [IF_NAMESIZE - 1] = '\0';
   if (mask && ioctl (fd, SIOCGIFNETMASK, &if_req) == 0)
-    *mask = g_strdup (inet_ntoa (((struct sockaddr_in *) &if_req.ifr_netmask)->sin_addr));
+    *mask = g_strdup (inet_ntoa (((struct sockaddr_in *) &if_req.ifr_addr)->sin_addr));
 
   close (fd);
 
@@ -775,16 +874,16 @@ static struct HwType
 } hw_types [] =
   {
 #ifdef ARPHRD_NETROM
-    { ARPHRD_NETROM, N_("AMPR NET/ROM Device"), print_ax25_addr },
+    { ARPHRD_NETROM, N_("AMPR NET/ROM"), print_ax25_addr },
 #endif
 #ifdef ARPHRD_ETHER
-    { ARPHRD_ETHER, N_("Ethernet Device"), print_mac_addr },
+    { ARPHRD_ETHER, N_("Ethernet"), print_mac_addr },
 #endif
 #ifdef ARPHRD_EETHER
     { ARPHRD_EETHER, NULL, NULL },
 #endif
 #ifdef ARPHRD_AX25
-    { ARPHRD_AX25, N_("AMPR AX.25 Device"), NULL },
+    { ARPHRD_AX25, N_("AMPR AX.25"), NULL },
 #endif
 #ifdef ARPHRD_PRONET
     { ARPHRD_PRONET, NULL, NULL },
@@ -793,58 +892,58 @@ static struct HwType
     { ARPHRD_CHAOS, NULL, NULL },
 #endif
 #ifdef ARPHRD_IEEE802
-    { ARPHRD_IEEE802, N_("16/4 Mbps Token Ring Device"), print_mac_addr },
+    { ARPHRD_IEEE802, N_("16/4 Mbps Token Ring"), print_mac_addr },
 #endif
 #ifdef ARPHRD_ARCNET
-    { ARPHRD_ARCNET, N_("ARCnet Device"), print_arcnet_addr },
+    { ARPHRD_ARCNET, N_("ARCnet"), print_arcnet_addr },
 #endif
 #ifdef ARPHRD_APPLETLK
     { ARPHRD_APPLETLK, NULL, NULL },
 #endif
 #ifdef ARPHRD_DLCI
-    { ARPHRD_DLCI, N_("Frame Relay DLCI Device"), print_dlci_addr },
+    { ARPHRD_DLCI, N_("Frame Relay DLCI"), print_dlci_addr },
 #endif
 #ifdef ARPHRD_ATM
     { ARPHRD_ATM, NULL, NULL },
 #endif
 #ifdef ARPHRD_METRICOM
-    { ARPHRD_METRICOM, N_("Metricom Starmode IP Device"), NULL },
+    { ARPHRD_METRICOM, N_("Metricom Starmode IP"), NULL },
 #endif
 #ifdef ARPHRD_SLIP
-    { ARPHRD_SLIP, N_("Serial Line IP Device"), NULL },
+    { ARPHRD_SLIP, N_("Serial Line IP"), NULL },
 #endif
 #ifdef ARPHRD_CSLIP
-    { ARPHRD_CSLIP, N_("VJ Serial Line IP Device"), NULL },
+    { ARPHRD_CSLIP, N_("VJ Serial Line IP"), NULL },
 #endif
 #ifdef ARPHRD_SLIP6
-    { ARPHRD_SLIP6, N_("6-bit Serial Line IP Device"), NULL },
+    { ARPHRD_SLIP6, N_("6-bit Serial Line IP"), NULL },
 #endif
 #ifdef ARPHRD_CSLIP6
-    { ARPHRD_CSLIP6, N_("VJ 6-bit Serial Line IP Device"), NULL },
+    { ARPHRD_CSLIP6, N_("VJ 6-bit Serial Line IP"), NULL },
 #endif
 #ifdef ARPHRD_RSRVD
     { ARPHRD_RSRVD, NULL, NULL },
 #endif
 #ifdef ARPHRD_ADAPT
-    { ARPHRD_ADAPT, N_("Adaptive Serial Line IP Device"), NULL },
+    { ARPHRD_ADAPT, N_("Adaptive Serial Line IP"), NULL },
 #endif
 #ifdef ARPHRD_ROSE
-    { ARPHRD_ROSE, N_("AMPR ROSE Device"), print_rose_addr },
+    { ARPHRD_ROSE, N_("AMPR ROSE"), print_rose_addr },
 #endif
 #ifdef ARPHRD_X25
-    { ARPHRD_X25, N_("Generic X.25 Device"), print_x25_addr },
+    { ARPHRD_X25, N_("Generic X.25"), print_x25_addr },
 #endif
 #ifdef ARPHRD_PPP
-    { ARPHRD_PPP, N_("Point-to-Point Protocol Device"), NULL },
+    { ARPHRD_PPP, N_("Point-to-Point Protocol"), NULL },
 #endif
 #ifdef ARPHRD_CISCO
     { ARPHRD_CISCO, NULL, NULL },
 #endif
 #ifdef ARPHRD_HDLC
-    { ARPHRD_HDLC, N_("(Cisco)-HDLC Device"), NULL },
+    { ARPHRD_HDLC, N_("(Cisco)-HDLC"), NULL },
 #endif
 #ifdef ARPHRD_LAPB
-    { ARPHRD_LAPB, N_("LAPB Device"), NULL },
+    { ARPHRD_LAPB, N_("LAPB"), NULL },
 #endif
 #ifdef ARPHRD_DDCMP
     { ARPHRD_DDCMP, NULL, NULL },
@@ -853,31 +952,31 @@ static struct HwType
     { ARPHRD_RAWHDLC, NULL, NULL },
 #endif
 #ifdef ARPHRD_TUNNEL
-    { ARPHRD_TUNNEL, N_("IPIP Tunnel Device"), NULL },
+    { ARPHRD_TUNNEL, N_("IPIP Tunnel"), NULL },
 #endif
 #ifdef ARPHRD_TUNNEL6
     { ARPHRD_TUNNEL6, NULL, NULL },
 #endif
 #ifdef ARPHRD_FRAD
-    { ARPHRD_FRAD, N_("Frame Relay Access Device Device"), NULL },
+    { ARPHRD_FRAD, N_("Frame Relay Access Device"), NULL },
 #endif
 #ifdef ARPHRD_SKIP
     { ARPHRD_SKIP, NULL, NULL },
 #endif
 #ifdef ARPHRD_LOOPBACK
-    { ARPHRD_LOOPBACK, N_("Local Loopback Device"), print_mac_addr },
+    { ARPHRD_LOOPBACK, N_("Local Loopback"), print_mac_addr },
 #endif
 #ifdef ARPHRD_LOCALTLK
     { ARPHRD_LOCALTLK, NULL, NULL },
 #endif
 #ifdef ARPHRD_FDDI
-    { ARPHRD_FDDI, N_("Fiber Distributed Data Interface Device"), print_mac_addr },
+    { ARPHRD_FDDI, N_("Fiber Distributed Data Interface"), print_mac_addr },
 #endif
 #ifdef ARPHRD_BIF
     { ARPHRD_BIF, NULL, NULL },
 #endif
 #ifdef ARPHRD_SIT
-    { ARPHRD_SIT, N_("IPv6-in-IPv4 Device"), NULL },
+    { ARPHRD_SIT, N_("IPv6-in-IPv4"), NULL },
 #endif
 #ifdef ARPHRD_IPDDP
     { ARPHRD_IPDDP, NULL, NULL },
@@ -889,16 +988,16 @@ static struct HwType
     { ARPHRD_PIMREG, NULL, NULL },
 #endif
 #ifdef ARPHRD_HIPPI
-    { ARPHRD_HIPPI, N_("HIPPI Device"), print_mac_addr },
+    { ARPHRD_HIPPI, N_("HIPPI"), print_mac_addr },
 #endif
 #ifdef ARPHRD_ASH
-    { ARPHRD_ASH, N_("Ash Device"), print_ash_addr },
+    { ARPHRD_ASH, N_("Ash"), print_ash_addr },
 #endif
 #ifdef ARPHRD_ECONET
-    { ARPHRD_ECONET, N_("Econet Device"), print_econet_addr },
+    { ARPHRD_ECONET, N_("Econet"), print_econet_addr },
 #endif
 #ifdef ARPHRD_IRDA
-    { ARPHRD_IRDA, N_("IrLAP Device"), print_irda_addr },
+    { ARPHRD_IRDA, N_("IrLAP"), print_irda_addr },
 #endif
 #ifdef ARPHRD_FCPP
     { ARPHRD_FCPP, NULL, NULL },
@@ -913,7 +1012,7 @@ static struct HwType
     { ARPHRD_FCPFABRIC, NULL, NULL },
 #endif
 #ifdef ARPHRD_IEEE802_TR
-    { ARPHRD_IEEE802_TR, N_("16/4 Mbps Token Ring Device"), print_mac_addr },
+    { ARPHRD_IEEE802_TR, N_("16/4 Mbps Token Ring"), print_mac_addr },
 #endif
 #ifdef ARPHRD_IEEE80211
     { ARPHRD_IEEE80211, NULL, NULL },
@@ -925,6 +1024,7 @@ netstatus_iface_get_hw_details (NetstatusIface  *iface,
 				char           **hw_addr)
 				
 {
+#ifdef SIOCGIFHWADDR
   static struct HwType *hw_type = NULL;
   struct ifreq          if_req;
   int                   fd;
@@ -973,6 +1073,10 @@ netstatus_iface_get_hw_details (NetstatusIface  *iface,
     *hw_addr = hw_type->print_hw_addr (if_req.ifr_hwaddr.sa_data);
 
   return hw_type;
+
+#else /* !defined(SIOCGIFHWADDR) */
+  return NULL;
+#endif 
 }
 
 gboolean
@@ -1017,6 +1121,12 @@ netstatus_iface_get_device_details (NetstatusIface  *iface,
   return TRUE;
 }
 
+#if !defined(HAVE_SOCKADDR_SA_LEN)
+#define NETSTATUS_SA_LEN(saddr) (sizeof (struct sockaddr))
+#else
+#define NETSTATUS_SA_LEN(saddr) (MAX ((saddr)->sa_len, sizeof (struct sockaddr)))
+#endif /* HAVE_SOCKADDR_SA_LEN */
+
 /* Taken From R. Stevens Unix Network Programming Vol. 1.
  *
  * SIOCGIFCONF does not return an error on all systems if
@@ -1083,8 +1193,8 @@ netstatus_list_interface_names (GError **error)
   struct ifconf *if_conf;
   GList         *interfaces;
   GList         *loopbacks;
+  char          *p;
   int            fd;
-  int            i;
   
   if ((fd = socket (AF_INET, SOCK_DGRAM, 0)) < 0)
     {
@@ -1105,12 +1215,14 @@ netstatus_list_interface_names (GError **error)
   interfaces = NULL;
   loopbacks  = NULL;
 
-  for (i = 0; i < if_conf->ifc_len / sizeof (struct ifreq); i++)
+  for (p = if_conf->ifc_buf; p < if_conf->ifc_buf + if_conf->ifc_len;)
     {
-      struct ifreq if_req = if_conf->ifc_req [i];
-      gboolean     loopback = FALSE;
+      struct ifreq *if_req = (struct ifreq *) p;
+      gboolean      loopback = FALSE;
 
-      if (ioctl (fd, SIOCGIFFLAGS, &if_req) < 0)
+      p += sizeof (if_req->ifr_name) + NETSTATUS_SA_LEN (&if_req->ifr_addr);
+
+      if (ioctl (fd, SIOCGIFFLAGS, if_req) < 0)
 	{
 	  if (error)
 	    *error = g_error_new (NETSTATUS_ERROR,
@@ -1120,13 +1232,15 @@ netstatus_list_interface_names (GError **error)
 	}
       else
 	{
-	  loopback = (if_req.ifr_flags & IFF_LOOPBACK);
+	  loopback = (if_req->ifr_flags & IFF_LOOPBACK);
 	}
 
       if (!loopback)
-	interfaces = g_list_prepend (interfaces, g_strdup (if_req.ifr_name));
+	interfaces = netstatus_list_insert_unique (interfaces,
+						   g_strdup (if_req->ifr_name));
       else
-	loopbacks  = g_list_prepend (loopbacks,  g_strdup (if_req.ifr_name));
+	loopbacks  = netstatus_list_insert_unique (loopbacks,
+						   g_strdup (if_req->ifr_name));
     }
 
   interfaces = g_list_concat (interfaces, loopbacks);
